function results = dclde2018multisite(truthfiles, detectionfiles, varargin)
% results = dclde2018multisite(gt, detections, OptionalArgs)
% Similar to dclde2018score, but designed for multiple projects and sites
% Reads all data, splits into project/site specific results and 
% also returns combined results
%
% gt - ground truthed detections
% detections - detections to be evaluated
% gt and detections should either be:
%   filenames - Assumes that these are DCLDE 2018 format CSV files
%       and reads the files via read_annotations
%   structures - Structure format must be the same as that returned
%       by read_annotations (see read_annotations for details)
%
% Optional arguments
%     'IgnoreLabels', true|false (default false)
%        if true, only detection times are considered.  
%        Species and call names are ignored.  This can be useful for
%        testing signal detection algorithms.
%     'RemoveZeroDuration', true|false (default true)
%         Remove detections that have a duration of 0 s
%     'SpecificLabels', mapObj
%        Only consider detections from specified species (and possibly
%        specific calls from that species) when scoring.  All other
%        detections are not scored.  See read_annotations for details.
%
% CAVEATS:
%   Optional arguments RemoveZeroDuration and SpecificLabels are
%   only processed when gt and/or detections contain CSV filenames as
%   this functionality is implemented in the CSV reader function
%   read_annotations.
%
%   Degenerate cases of precision and recall return 1 which can be
%   misleading.  Examples:
%     Nothing to detect:  Recall = 1
%     Nothing detected:  Precision = 1
%
% See also:  dclde2018score, read_annotations
%
% Author:  Marie A. Roch, San Diego State University

narginchk(2,inf);

vidx = 1;
IgnoreLabelsArgs = {};  % default behavior
ScoreLabels = [];
while vidx <= length(varargin)
    switch varargin{vidx}
        case 'IgnoreLabels'
            IgnoreLabelsArgs = varargin(vidx:vidx+1);
            varargin(vidx:vidx+1) = [];  % Remove processed arg
        otherwise
            vidx = vidx+2;  % skip argument
    end
end

% ground-truth labeled detections
if isstruct(truthfiles)
    truth = truthfiles;
else
    truth = read_annotations(truthfiles, varargin{:});
end
% detections to be scored
if ~ isstruct(detectionfiles)
    detections = read_annotations(detectionfiles, varargin{:});
end

% partition ground truth and detections
[ptruth, pdetections] = projectsitesplit(truth, detections);
% Still need to think about empty cases
for idx = 1:length(ptruth)
    results(idx) = dclde2018score(ptruth(idx), pdetections(idx), ...
        IgnoreLabelsArgs{:});
end

% Add project/site information
for idx = 1:length(ptruth);
    results(idx).project = ptruth(idx).project{1};
    results(idx).site = ptruth(idx).site{1};
    % Copy ground truth and detections as user may wish to analyze
    results(idx).truth = ptruth(idx);
    results(idx).detections = pdetections(idx);
end

% Merge data
last = results(1);  % ensure fields in same order, we'll overwrite
last.project = 'all';
last.site = 'all';
% Merge statistics arrays
names = fieldnames(results(1));
% no concatenation of associations matrices
names = setdiff(names, 'associations');  
for idx = 1:length(names)
    if isvector(results(1).(names{idx})) ...
            && isnumeric(results(1).(names{idx})) ...
            && ~isscalar(results(1).(names{idx}))
        last.(names{idx}) = vertcat(results.(names{idx}));
    end
end

% Merge association matrices
% Get dimensions of assocation matrices
associationsN = zeros(length(results),2);
for idx = 1:length(results)
    associationsN(idx,:) = size(results(idx).associations);
end
% find number of rows/cols as we merge in nth matrix
associationsCum = cumsum(associationsN);

lastrow = 0;
lastcol = 0;
allrows = [];
allcols = [];
allvals = [];
for idx = 1:length(results)
    % stopped here, need to build extended, r, c, values, preallocate...
    [r,c,val] = find(results(idx).associations);
    % Adjust rows and columns
    r = r + lastrow;
    c = c + lastcol;
    % More efficient to preallocate, but not a huge deal for datasets
    % of the size we are dealing with.    
    allrows = [allrows; r];
    allcols = [allcols; c];
    allvals = [allvals; val];
    % Next matrix starts cumsum offsets into the sparse matrix
    lastrow = associationsCum(idx, 1);
    lastcol = associationsCum(idx, 2);
end  
last.associations = sparse(allrows, allcols, allvals, ...
    associationsCum(end,1), associationsCum(end,2));

last.detections = detections;
last.truth = truth;

% Compute merged statistics
last.truthCoverageOverallPct = sum(last.truthCoverage) / sum(truth.durations);
last.detectionCoverageOverallPct = sum(last.detectionCoverage) / ...
    sum(detections.durations);
detectionsI = last.fragmentation > 0;
last.Efragmentation = mean(last.fragmentation(detectionsI));
% Merge all false positive indicators
falsePosI = vertcat(results.falsePosI);
if length(falsePosI)
    last.precision = sum(~falsePosI)/length(falsePosI);
else    
    last.precision = 1;  % nothing detected, 100% precise
end
if last.truth.N
    last.recall = sum(last.truthCoverage > 0) / last.truth.N;
else
    last.recall = 1;  % Nothing to detect, recall must be perfect.
end
results(end+1) = last;

1;
